#!/usr/bin/python

##################################################################################
#
#	Copyright 2005 Larry Barber and the US Dept. of Agriculture.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#####################################################################################



import os
import sys
import popen2
import getopt	
import re
import time
import datetime 

import ConfigFile

global cfg_file
global tmp_dir

def getHosts():
	xymonhome = cfg_file.getVal("xymonhome", "/usr/local/xymon/server")
	xymongrep = cfg_file.getVal("xymongrep", "/usr/local/xymon/server/bin/xymongrep --noextras")
	chain_tag = cfg_file.getVal("chain_tag", "sslchain*")
	net = cfg_file.getVal("network")
	#print "net = " + net
	#print "xymonhome = " + xymonhome
	#print "xymongrep = " + xymongrep
	#print "chain_tag = " + chain_tag
	cmd = "XYMONHOME=" + xymonhome + " " + "XYMONNETWORK=" + net + " " + xymongrep + " " + chain_tag 
	#print "cmd = " + cmd
	r,w,e = popen2.popen3(cmd)
	hosts = {} 
	for l in r.readlines():
		#print "l = " + l
		dispname = l.split()[1]
		hosts[dispname] = []
		tag = l.split()[3]
		if tag.find('=') != -1:
			hostnames = tag.split('=')[1]
			for hp in hostnames.split(','):
				#print "hp = " + hp
				if hp.find(':') != -1:
					hostname,port=hp.split(':')
				else:
					hostname=hp
					port="443"
				hosts[dispname].append((hostname, port))
		else:
			hostname = dispname
			port="443"
			hosts[dispname].append((hostname, port))
	return hosts

def getCertList(hosts):
	getcerts_cmd = cfg_file.getVal("getcerts_cmd", "/usr/bin/openssl")
	getcerts_args = cfg_file.getVal("getcerts_args", " s_client -showcerts -connect ")

	for dispname in hosts.keys():
		for hp in hosts[dispname]:
			hostname = hp[0]
			port = hp[1]
			cmd = "echo quit | " + getcerts_cmd + " " + getcerts_args + " " + hostname + ":" + port + " > " + tmp_dir + "/" + hostname + "." + port + ".cert"
			#print "cmd = " + cmd
			r,w,e = popen2.popen3(cmd)
			#time.sleep(openssl_timeout)
			w.write('quit')
			#print "sent interrupt"
	return None

def getCerts(hlist):
	begin_pattern = cfg_file.getVal("begin_pattern", "[-]+BEGIN CERTIFICATE[-]+")
	end_pattern = cfg_file.getVal("end_pattern", "[-]+END CERTIFICATE[-]+")
	begin_re = re.compile(begin_pattern)
	end_re = re.compile(end_pattern)

	cert_dict={}
	for host,port in hlist:
		certfile = file (tmp_dir + '/' + host + "." + port + '.cert' , "r")
		in_cert = False
		cert_list=[]
		for l in certfile.readlines():
			#print l.strip()
			if in_cert == True:
				cert_line.append(l)
				if end_re.match(l):
					#print "found cert end"
					certificate = "".join(cert_line)
					#print "certificate = " + certificate
					cert_list.append(certificate)
					in_cert = False

			if begin_re.match(l):
				#print "found cert beginning"
				in_cert = True
				cert_line = []
				cert_line.append(l)
		cert_dict[(host,port)]=cert_list

	return cert_dict

def decodeCerts(cert_dict):
	readcerts_cmd = cfg_file.getVal("readcerts_cmd", "/usr/bin/openssl ")
	readcerts_args = cfg_file.getVal("readcerts_args", "x509 -text")

	cert_headers_dict = {}
	for host,port in cert_dict.keys():
		cert_headers_dict[(host,port)]=[]
		certs = cert_dict[(host,port)]
		for c in certs:
			cmd = "echo '" + c + "' | " + readcerts_cmd + " " + readcerts_args
			#print "c = " + c
			#print "cmd = " + cmd
			r,w,e = popen2.popen3(cmd)
			w.write(c + "\n")
			line_cnt =0
			header_lines = []
			for l in r.readlines():
				#print l.strip()
				if line_cnt < 12:
					header_lines.append(l)
					line_cnt += 1
				else:
					#print "header_lines = " + `header_lines`
					cert_headers_dict[(host,port)].append(header_lines)
					break
	return cert_headers_dict

def getStatus(cert_headers_dict):
	expiry_date_pattern = cfg_file.getVal("expiry_date_pattern", ".*Not After[ ]*:(.*)")
	expiry_re = re.compile(expiry_date_pattern)
	warn_threshold = int(cfg_file.getVal("warn_threshold", "30"))
	crit_threshold = int(cfg_file.getVal("crit_threshold", "14"))
	month_nums = { 'Jan':1, 'Feb':2, 'Mar':3, 'Apr':4, 'May':5, 'Jun':6, 'Jul':7, 'Aug':8, 'Sep':9, 'Oct':10, 'Nov':11, 'Dec':12 }

	statii_dict = {} 
	for h,p in cert_headers_dict.keys():
		statii_dict[(h,p)] = []
		for header in cert_headers_dict[(h,p)]:
			#print "header = " + `header`
			for l in header:
				m = expiry_re.match(l)
				if m:
					#print "m.group(1) = " + m.group(1)
					mon,day,time,year,tz = m.group(1).split()
					hour,min,sec = time.split(':')
					#print "mon,day,time,year,tz = " + mon + " " + day + " " + time + " " + year + " " + tz
					#print "hour,min,sec = " + hour + " " + min + " " + sec
					exdate = datetime.date(int(year), month_nums[mon], int(day))
					extime = datetime.time(int(hour), int(min), int(sec))
					fulldate = datetime.datetime.combine(exdate, extime)
		
					days_til_expiry = fulldate - datetime.datetime.today()
					#print "days_til_expiry.days = " + str(days_til_expiry.days)
	
					status = 'green'
					if days_til_expiry.days < crit_threshold:
						status = 'red'
					elif days_til_expiry.days < warn_threshold:
						status = 'yellow'
			statii_dict[(h,p)].append((header, days_til_expiry.days, status))	

	return statii_dict

def getOverallStatus(statii_dict):
	stat = 'green'
	#print "\n\nstatii = " + `statii`
	for h,p in statii_dict.keys():
		for s in statii_dict[(h,p)]:
			#print "s[2] = " + s[2]
			if stat == 'green':
				if s[2] == 'red':
					#print "changing from green to red"
					stat = 'red'
				elif s[2] == 'yellow':
					#print "changing from green to yellow"
					stat = 'yellow'
			elif stat == 'yellow':
				if s[2] == 'red':
					#print "changing from yellow to red"
					stat = 'red'
	#print "returning = " + stat
	return stat

def getHostStatus(statii):
	stat = 'green'
	#print "\n\nstatii = " + `statii`
	for s in statii:
		#print "s[2] = " + s[2]
		if stat == 'green':
			if s[2] == 'red':
				#print "changing from green to red"
				stat = 'red'
			elif s[2] == 'yellow':
				#print "changing from green to yellow"
				stat = 'yellow'
		elif stat == 'yellow':
			if s[2] == 'red':
				#print "changing from yellow to red"
				stat = 'red'
	#print "returning = " + stat
	return stat

def sendResults(dname, statii_dict):
	xymonhome = cfg_file.getVal("xymonhome", "/usr/local/xymon/server")
	xymoncmd = cfg_file.getVal("xymoncmd","/usr/local/xymon/server/bin/xymon")
	xymonhost = cfg_file.getVal("xymonhost","peregrin")
	xymontest = cfg_file.getVal("xymontest","sslchain")
	status_lifetime = cfg_file.getVal("status_lifetime","1500")

	overall_stat = getOverallStatus(statii_dict)
	#print "overall_stat = " + overall_stat

	#print "dname = " + `dname`
	xymon_cmd_list = []
	xymon_cmd_list.append(xymoncmd + " " +  xymonhost + " 'status+" + status_lifetime + " " + dname + "." +  xymontest + " " + overall_stat + "\n")
	xymon_cmd_list.append("&" + overall_stat + " <font size=+2><b> " + time.asctime() + "</b> Certificates for " + dname + " are in ") 
	if overall_stat == 'red':
		xymon_cmd_list.append('<b><font color=red>critical</font></b> ')
	elif overall_stat == 'yellow':
		xymon_cmd_list.append('<b><font color=yellow>guarded</font></b> ')
	elif overall_stat == 'green':
		xymon_cmd_list.append('<b><font color=green>good</font></b> ')
	xymon_cmd_list.append("condition </font><br><br>")

	for h,p in statii_dict.keys():
		host_stat = getHostStatus(statii_dict[(h,p)])
		xymon_cmd_list.append('&' + host_stat + " <font size=+2> Certificate chain for " + h + ":" + p + " is in ")
		if host_stat == 'red':
			xymon_cmd_list.append('<b><font color=red>critical</font></b> ')
		elif host_stat == 'yellow':
			xymon_cmd_list.append('<b><font color=yellow>guarded</font></b> ')
		elif host_stat == 'green':
			xymon_cmd_list.append('<b><font color=green>good</font></b> ')
		xymon_cmd_list.append("condition </font><br><br>")
		xymon_cmd_list.append("<table>")

		for s in statii_dict[(h,p)]:
			xymon_cmd_list.append("<tr><td>&nbsp;&nbsp;&nbsp;</td>")
			xymon_cmd_list.append("<td>&" + s[2] + "</td>")
			xymon_cmd_list.append("<td> Certificate will expire in " + str(s[1]) + " days</td></tr>")
			xymon_cmd_list.append("<tr><td>&nbsp;&nbsp;&nbsp;</td><td></td><td>" + "".join(s[0]) + "</td></tr>")

		xymon_cmd_list.append("</table><br><br>")
	xymon_cmd_list.append("'")

	xymon_cmd = "".join(xymon_cmd_list)
	#print "xymon_cmd = " + xymon_cmd

	r,w,e = popen2.popen3(xymon_cmd)

	return None

def cleanup(hosts):
	for dname in hosts.keys():
		#print "hosts[dname] = " + `hosts[dname]`
		for hname,port in hosts[dname]:
			fname = tmp_dir + "/" + hname + "." + port + '.cert' 
			if os.path.isfile(fname):
				#print "removing file " + fname
				os.remove(fname)
				#print "removed file " + fname
	return None

opts, args = getopt.getopt(sys.argv[1:], "c:")
cfg_file=None
for o,a in opts:
	if o == '-c':
		config_filename=a
		cfg_file=ConfigFile.ConfigFile(config_filename, separator="::")

if cfg_file==None:
	sys.stderr.write("Must specify name of configuration file on command line\n")
	sys.exit(-1)


tmp_dir = cfg_file.getVal("tmp_dir","/usr/local/xy5mon/server/tmp")
#xymonhome = cfg_file.getVal("xymonhome", "/usr/local/xymon/server")

hosts = getHosts()
#hosts = [('gummite')]
print "hosts = " + `hosts`
getCertList(hosts)
time.sleep(2)

for dname in hosts.keys():
	print 'dname = ' + dname
	hlist = hosts[dname]	
	#print 'hlist = ' + `hlist`
	if len(hlist) == 0:
		continue
	cert_dict = getCerts(hlist)
	#print "cert_dict = " + `cert_dict`
	cert_headers_dict = decodeCerts(cert_dict)
	#for h,p in cert_headers_dict.keys():
		#print "h = " + h 
		#print "p = " + p 
		#for header in cert_headers_dict[(h,p)]:
			#print "header = " + `header`
			#for l in header:
				#print "l = " + l
	statii_dict = getStatus(cert_headers_dict)
	#for h,p in statii_dict.keys():
	#	print "h = " + h 
	#	print "p = " + p 
	#	for hdr in statii_dict[(h,p)]:
	#		print "hdr = " + `hdr`
	sendResults(dname, statii_dict)

cleanup(hosts)

sys.exit(0)
	
